@@ -30,6 +30,10 @@
 
 
 
+char *liqclip=NULL;
+
+
+
 #define XK_MISCELLANY
 
 #include "X11/keysymdef.h"
@@ -41,65 +45,14 @@
 #define  GREEN    "rgb(0,173,0)"
 #define  BLUE     "rgb(0,0,255)"
 #define  YELLOW   "rgb(225,225,0)"
-#define  CYAN     "rgba(0,175,175,128)"
+#define  CYAN     "rgb(0,175,175)"
 #define  MAGENTA  "rgb(255,0,255)"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 
-int textbox_selectall(liqcell *textbox)
-{
-	// select all!
-	int selstart = liqcell_propgeti(  textbox,"selstart",-1);
-	int sellength = liqcell_propgeti(  textbox,"sellength",0);
-	int cursorpos = liqcell_propgeti(  textbox,"cursorpos",-1);
-	
-	char *caption = liqcell_getcaption(textbox);
-	int captionlen = strlen(caption);
-
-
-	liqcell_propseti(  textbox,  "selstart",  0 );
-	liqcell_propseti(  textbox,  "sellength", captionlen );
-	liqcell_propseti(  textbox,  "cursorpos", captionlen );
-       
-	
-}
-
-int textbox_fakebackspace(liqcell *textbox)
-{
-	// fake a backspace!
-	liqcellkeyeventargs keyargs={0};
-	keyargs.keycode = (int)8;
-	char delbuf[2];
-	delbuf[0]=8;
-	delbuf[1]=0;
-	snprintf(keyargs.keystring,sizeof(keyargs.keystring),delbuf);
-	keyargs.ispress = 1;
-	liqcell_handlerrun(textbox,"keypress",&keyargs);
-}
-
-
-
-
-int textbox_clear(liqcell *textbox)
-{
-	// clear
-
-	liqcell_setcaption(textbox,"");
-
-
-	liqcell_propseti(  textbox,  "selstart",  0 );
-	liqcell_propseti(  textbox,  "sellength", 0 );
-	liqcell_propseti(  textbox,  "cursorpos", 0 );
-       
-	return 0;
-}
-
-
-
-
-	// damn, this should be taken care of by the OS itself
-	// this is because the hitpoint will not always match with where I think it will be
-	// this is bad and good in practice
 	static int textbox_mouse(liqcell *self, liqcellmouseeventargs *args,void *context)
 	{
 		// if i have a font on my cell, surely it will have been rendered correctly first..
@@ -128,9 +81,80 @@ int textbox_clear(liqcell *textbox)
 		//int caplen = strlen(cap);
 		
 		
-		int mx = args->mex - liqcell_getx(self) - 2;
 		
-		int chpos = liqfont_textfitinside(font,cap,mx);
+		
+		//##########################
+		// this tells me where abouts I am on a single line
+		int mx = args->mex - liqcell_getx(self) - 8;		// available space per line
+		int my = args->mey - liqcell_gety(self) - 4;
+		int chpos = liqfont_textfitinside(font,cap,mx);		// char pos of selection
+		
+		
+		//##########################
+		// count all the lines	
+		#define linemax 128
+		char *linestarts[linemax];
+		int lineheight = liqfont_textheight(font);
+		int lineoffsets[linemax];
+		int linelengths[linemax];
+		int linecount=0;	
+		char *c = cap;
+		while(c && *c)
+		{
+			int tl = strlen(c);		// total length remaining
+			int lc = liqfont_textfitinside(font, c, liqcell_getw(self)-4 );
+					if(lc<tl)
+					{
+						int le = lc;
+						while(le>0)
+						{
+							switch(c[le-1])
+							{
+								case ' ':
+								case ',':
+								case ';':
+								case ':':
+								case '.':
+									le--;
+									break;
+								default:
+									goto fin;
+							}
+						}
+						fin:
+						if(le>0)lc=le;
+						while(c[lc]==' ')lc++;
+					}	
+			linestarts[linecount] = c;
+			lineoffsets[linecount] = c-cap;
+			linelengths[linecount] = lc;
+			linecount++;
+			c=&c[lc];
+		}
+		
+		
+		
+		
+		//##########################
+		// work out which line the mouse has hit
+		int ly=0;
+		
+		ly = my / lineheight;
+		//liqapp_log("mx=%3i, my=%3i, chpos=%3i   lx=%3i, ly=%3i",mx,my,chpos, lx,ly);
+		
+		if(ly>=linecount)ly=linecount-1;
+		
+		if(ly>=0)
+		{
+			// adjust chpos to make sure now we are selecting from the correct line
+			chpos = lineoffsets[ly] + liqfont_textfitinside(font, linestarts[ly] ,mx);
+		}
+		
+		
+		
+		
+		
+		
 										  
 		// neat :) i know where the mouse clicked (left aligned text only...)
 		// todo: handle other alignments
@@ -186,8 +210,8 @@ int textbox_clear(liqcell *textbox)
 		char *caption = liqcell_getcaption(self);
 		int captionlen = strlen(caption);
 		
-		char *key = args->keystring;
-		if(!key)key="";
+		char *key =strdup( args->keystring ? args->keystring : "");
+		//if(!key)key="";
 		if(*key && *key<32)
 		{
 			if(*key==10)
@@ -198,7 +222,7 @@ int textbox_clear(liqcell *textbox)
 				// Wed Aug 19 00:23:17 2009 lcuk : alternative is a default button as in visual basic
 				// Wed Aug 19 00:23:41 2009 lcuk : how do i find the related default button though within the system
 				liqcell_handlerrun(self,"keypress_enter",NULL);
-				key="";
+				free(key);	key=strdup("");
 			}
 			if(*key==8 || *key==9)
 			{
@@ -207,7 +231,7 @@ int textbox_clear(liqcell *textbox)
 			else
 			{
 				// ack! ignore these in single line textbox!
-				key="";
+				free(key);	key=strdup("");
 			}
 		}
 
@@ -274,7 +298,36 @@ int textbox_clear(liqcell *textbox)
 					liqcell_propseti(  self,  "selstart",  selstart);
 					liqcell_propseti(  self,  "sellength", sellength);
 					liqcell_propseti(  self,  "cursorpos", cursorpos);
-				}	
+				}
+				else
+				{
+				
+
+					if(args->keycode==120 && (args->keymodifierstate&4))
+					{
+						liqapp_log("key CTRL X :: CUT");
+						if(liqclip){free(liqclip);liqclip=NULL;}
+						liqclip = strndup(&caption[selstart],sellength);
+						free(key);	key=strdup(""); keylen = strlen(key);
+						goto selch;
+					}
+					if(args->keycode==99 && (args->keymodifierstate&4))
+					{
+						liqapp_log("key CTRL C :: COPY");
+						if(liqclip){free(liqclip);liqclip=NULL;}
+						liqclip = strndup(&caption[selstart],sellength);
+						free(key);
+						return 0;
+					}
+	
+					if(args->keycode==118 && (args->keymodifierstate&4))
+					{
+						liqapp_log("key CTRL V :: PASTE");
+						free(key); key=strdup(liqclip?liqclip:"");
+						keylen = strlen(key);
+						goto selch;
+					}
+				}
 
 			}
 			else
@@ -283,15 +336,15 @@ int textbox_clear(liqcell *textbox)
 				if(*key==8)
 				{
 					// delete ;)
-					key="";
-					keylen=0;
+					free(key);	key=strdup(""); keylen = strlen(key);
 					if(selstart>0 && sellength==0)
 					{
 						selstart--;
 						sellength++;
 					}
 				}
-			
+
+selch:	{ }		
 				//
 				//liqcell_setcaption(self,args->keystring);
 				char *aftersel=&caption[selstart+sellength];
@@ -302,6 +355,14 @@ int textbox_clear(liqcell *textbox)
 				// selstart+sellen..end
 				
 				
+				int maxlen = liqcell_propgeti(  self,"maxlength",0);
+				if(maxlen>0)
+				{
+					if( (selstart + keylen + aftersellen)>maxlen ) keylen = maxlen-(selstart+aftersellen);
+					if(keylen<0)keylen=0;
+				}
+				
+				
 				// !-- BUG FIX BY ZACH HABERSANG -- !
 				// ----------------------------------
 				// Program would segfault when 25 or so characters were entered
@@ -311,7 +372,7 @@ int textbox_clear(liqcell *textbox)
 				// 20090615_210659 lcuk : and me to explain why it was wrong ;) damn those +1 adjustments..
 				
 				int reqd = selstart  +  keylen  +  aftersellen + 1;
-				char *buff=malloc(reqd);
+				char *buff=(char *)malloc(reqd);
 				char *block=buff;
 				if(buff)
 				{
@@ -347,7 +408,7 @@ int textbox_clear(liqcell *textbox)
 			
 		}
 		
-		
+		if(key)free(key);
 		return 0;
 
 	}
@@ -361,27 +422,23 @@ static int textbox_keyrelease(liqcell *self, liqcellkeyeventargs *args)
 	return 0;
 }
 
-
-
-static int widget_click(liqcell *self, liqcellclickeventargs *args)
-{
-	liqcell *dialog=liqcell_getcontent(self);
-	liqcell_easyrun(dialog);
-	liqcell_setvisible(dialog,1);
-	//args->newdialogtoopen = liqcell_getcontent(self);
-	return 1;
-}
-
 static int keyboard_show_button_click(liqcell *self,liqcellclickeventargs *args, liqcell *textbox)
 {
+	if(	liqcell_propgeti(textbox,"vkbd_button_allowed",  1  ) == 0 ) return 1;
+	
 	liqcell *vkbd = liqcell_quickcreatevis("vkbd", "liqkeyboard", 0, 0, -1, -1);
 	liqcell *vkbd_textbox = liqcell_child_lookup(vkbd, "liqkeyboard_textbox");
 	
+	
+	
 	if(vkbd)
 	{
 		// 20090814_203451 lcuk : make sure the * follow into the vkb :)
 		liqcell_propseti(vkbd_textbox,"textispassword",  liqcell_propgeti(textbox,"textispassword",0)  );
 		
+		// disable usage as a default
+		liqcell_propseti(vkbd_textbox,"vkbd_button_allowed",  0  );
+		
 		char *caption = liqcell_getcaption(textbox);
 		liqcell_setcaption(vkbd_textbox, caption);
 		liqcell_setdata(vkbd, textbox);
@@ -429,44 +486,60 @@ liqcell *textbox_create()
 	{
 			
 		liqcell_setfont(   self,  liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
-		liqcell_propsets(  self,  "backcolor", "rgb(100,255,150)" );
-		liqcell_propsets(  self,  "textcolor", "rgb(20,30,40)" );
-		liqcell_propsets(  self,  "bordercolor", "rgb(255,255,255)" );
-		
+		//liqcell_propsets(  self,  "backcolor", "rgb(100,255,150)" );
+		//liqcell_propsets(  self,  "textcolor", "rgb(20,30,40)" );
+		//liqcell_propsets(  self,  "bordercolor", "rgb(255,255,255)" );
+
+        liqcell_propsets(  self,  "textcolor", "rgb(0,0,0)" );
+		//liqcell_setimage(  self,  liqimage_cache_getfile("/usr/share/liqbase/onedotzero/media/2_text_back.png", 0,0,1) );
+        //liqcell_setimage(  self,  liqimage_cache_getfile("/usr/share/liqbase/onedotzero/media/2.message/message_input_field.png", 0,0,1) );
+		liqcell_propseti(  self,  "lockaspect",  0 );
+		//liqcell_propremoves(  self,  "bordercolor" );
+		//liqcell_propremoves(  self,  "backcolor" );
+				
+		liqcell_propseti(  self,  "maxlength",  140 );
 
 		liqcell_propseti(  self,  "selstart",  0 );
 		liqcell_propseti(  self,  "sellength", 0 );
 		liqcell_propseti(  self,  "cursorpos", 0 );
 			
-		liqcell_handleradd(self,    "mouse",      textbox_mouse);
-		liqcell_handleradd(self,    "keypress",   textbox_keypress);
-		liqcell_handleradd(self,    "keyrelease", textbox_keyrelease);
+		liqcell_handleradd(self,    "mouse",      (void*)textbox_mouse);
+		liqcell_handleradd(self,    "keypress",   (void*)textbox_keypress);
+		liqcell_handleradd(self,    "keyrelease", (void*)textbox_keyrelease);
 		
-		liqcell_handleradd(self,    "resize",   textbox_resize);
+		liqcell_handleradd(self,    "resize",   (void*)textbox_resize);
 	
-			
+		// portrait mode, make the textbox click go here as well
+		liqcell_handleradd_withcontext(self, "click", (void*)keyboard_show_button_click, self);
+		// this is potentially wrong, but its worth to see
+
+		
 		// add vkbd
 		liqcell *vkbd_command = liqcell_quickcreatevis("vkbd_command" , "commandbutton", 0, 0, 0, 0);
-		liqcell_handleradd_withcontext(vkbd_command, "click", keyboard_show_button_click, self);
-		liqcell_setfont(vkbd_command, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (12), 0));
-		liqcell_setcaption(vkbd_command, "ABC");
+		liqcell_handleradd_withcontext(vkbd_command, "click", (void*)keyboard_show_button_click, self);
+		//liqcell_setfont(vkbd_command, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (12), 0));
+		//liqcell_setcaption(vkbd_command, "ABC");
 		liqcell_propsets(vkbd_command, "backcolor", CYAN);
 		liqcell_propsets(vkbd_command, "textcolor", BLACK);
 		liqcell_propseti(vkbd_command, "textalign", 2);
 		liqcell_propseti(vkbd_command, "textaligny", 2);
 		liqcell_propseti(vkbd_command, "lockaspect", 1);
+				
+		liqcell_setvisible(vkbd_command,0);		// Wed Aug 19 19:08:20 2009 lcuk : proper way would be checking for no keyboard..
+		// 20100702: lcuk: reenabled by default for portrait test.
+		// thank you zack :)
 		
-		if( strcasecmp("RX-34", liqapp_hardware_product_get() ) ==0 )
-		{
-			// keep visible
-		}
-		else
-		{
-			// hide
-			liqcell_setvisible(vkbd_command,0);
-		}
+		// actually, removing button, but retaining action :)
+		
+		liqcell_propseti(self,"vkbd_button_allowed",  1  );
+
 		
 		liqcell_child_insert(self, vkbd_command);
 	}
 	return self;
 }
+
+#ifdef __cplusplus
+}
+#endif
+
